"""Base/interface of each module."""
from abc import ABC
from PySide2 import QtGui

# Font for module headline
HEADLINE_FONT = QtGui.QFont()
HEADLINE_FONT.setFamily("Arial")
HEADLINE_FONT.setPointSize(12)
HEADLINE_FONT.setWeight(100)


class ModuleBase(ABC):
    """Interface for SOI-modules."""

    type = None

    def __init__(self):
        if self.type is None:
            raise NotImplementedError

    def get_size(self):
        """Abstract method, should be implemented by derived class."""
        raise NotImplementedError

    def get_data(self):
        """Abstract method, should be implemented by derived class."""
        raise NotImplementedError

    @staticmethod
    def get_user_friendly_name():
        """Abstract method, should be implemented by derived class."""
        raise NotImplementedError

    @staticmethod
    def get_icon():
        """Abstract method, should be implemented by derived class."""
        raise NotImplementedError


def resize_table(widget, resize_row=True, resize_column=True):
    """Calculate and set the size of a QTableWidget.

    Parameters
    ----------
    widget : QTableWidget
        QTablewidget-instance to calculate and set size.
    resize_row : bool
        Resizes rows to contents if True, by default True.
    resize_column : bool
        Resizes columns to contents if True, by default True.
    """
    if resize_row:
        widget.resizeRowsToContents()
    if resize_column:
        widget.resizeColumnsToContents()

    widget.resizeColumnToContents(2)
    width, height = get_table_size(widget)

    headline = widget.item(0, 0).text()
    fm = QtGui.QFontMetrics(HEADLINE_FONT)
    headline_width = fm.width(headline) + 10  # + 10 offset

    # If headline is wider than widget
    if width < headline_width:
        difference = headline_width - width
        width += difference
        old_width = widget.columnWidth(2)
        widget.setColumnWidth(2, old_width + difference)

    widget.setFixedWidth(width)
    widget.setFixedHeight(height)


def get_table_size(widget):
    """Calculate and return total width and height of a QTableWidget.

    Parameters
    ----------
    widget : QTableWidget
        QTableWidget-instance to calculate and return size of.

    Returns
    -------
    Tuple
        Total (width, height)
    """
    # Calculate total width and height of columns and rows
    width = 0
    height = 0

    for i in range(widget.columnCount()):
        width += widget.columnWidth(i) + 0.5

    for i in range(widget.rowCount()):
        height += widget.rowHeight(i) + 0.5

    return width, height


def set_module_pos(widget, pos):
    """Set position of module (widget).

    Parameters
    ----------
    widget : QWidget
        Widget to move.
    pos : QPoint
        Position (x, y).
    """
    widget.move(pos)
